// HPC++ Run Time Sytem Library
// Extreme! Computing Lab - Computer Science Department
// Copyright (c) 1997 Indiana University
// Authors: Pete Beckman, Dennis Gannon, Todd Green, and Liz Johnson
//
// $Id: HPCxx_BufferImpl.C,v 1.4 1999/07/22 20:56:57 sdiwan Exp $

#include <HPCxx_Profile.h>
#include <stdio.h>
#include <iostream.h>
#include <HPCxx_BufferImpl.h>

const int HPCXX_NO_BUFFER_SPACE = -1;

HPCxx_BufferImpl::HPCxx_BufferImpl(int bufSize)
{
 TAU_PROFILE("HPCxx_BufferImpl::HPCxx_BufferImpl", "void (int)", TAU_HPCXX);

 buffer_ = new globus_nexus_buffer_t;
 destroyFlag = true; // SMD
 if( globus_nexus_buffer_init(buffer_, bufSize, 0)){
   exit(0);
   }
}

HPCxx_BufferImpl::HPCxx_BufferImpl(globus_nexus_buffer_t &buffer)
  : buffer_(&buffer),  destroyFlag(false) // SMD
{
  TAU_PROFILE("HPCxx_BufferImpl::HPCxx_BufferImpl", "void (globus_nexus_buffer_t )", 
    TAU_HPCXX);
}

HPCxx_BufferImpl::~HPCxx_BufferImpl()
{
  TAU_PROFILE("HPCxx_BufferImpl::~HPCxx_BufferImpl", "void ()", TAU_HPCXX);
  if (destroyFlag) { // SMD
    delete buffer_;
  }
}

int HPCxx_BufferImpl::pack(const HPCxx_ContextID* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (HPCxx_ContextID*, int)", 
	      TAU_HPCXX);
  int i;
  globus_nexus_startpoint_t *theSPs = new globus_nexus_startpoint_t[count];
  for(i = 0; i < count; i++)
    globus_nexus_startpoint_copy(&(theSPs[i]), &( ((HPCxx_ContextID*) data)[i] ));
  int err = globus_nexus_check_buffer_size(buffer_,
					   globus_nexus_sizeof_startpoint(theSPs,count),
					   globus_nexus_sizeof_startpoint(theSPs,count),
					   0,
					   0);
  if (err < 0) {
    printf("pack error\n");
    return(HPCXX_NO_BUFFER_SPACE);
  }
  globus_nexus_put_startpoint_transfer(buffer_, theSPs, count);
  delete [] theSPs;
  
  return(HPCXX_SUCCESS);
}

int HPCxx_BufferImpl::pack(const float* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (float*, int)", TAU_HPCXX);
  int err = globus_nexus_check_buffer_size(buffer_,
				    globus_nexus_sizeof_float(count),
				    globus_nexus_sizeof_float(count),
				    0,
				    0);
  if (err < 0) {
      printf("pack error\n");
      return(HPCXX_NO_BUFFER_SPACE);
  }
  globus_nexus_put_float(buffer_, (float*) data, count);

  return(HPCXX_SUCCESS);
}

int HPCxx_BufferImpl::pack(const double* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (double*, int)", TAU_HPCXX);
  int err = globus_nexus_check_buffer_size(buffer_,
                                    globus_nexus_sizeof_double(count),
                                    globus_nexus_sizeof_double(count),
                                    0,
                                    0);
  if (err < 0) return(HPCXX_NO_BUFFER_SPACE);

  globus_nexus_put_double(buffer_, (double*) data, count);

  return(HPCXX_SUCCESS);
}

int HPCxx_BufferImpl::pack(const short* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (short*, int)", TAU_HPCXX);
  int err = globus_nexus_check_buffer_size(buffer_,
                                    globus_nexus_sizeof_short(count),
                                    globus_nexus_sizeof_short(count),
                                    0,
                                    0);
  if (err < 0) return(HPCXX_NO_BUFFER_SPACE);

  globus_nexus_put_short(buffer_, (short*) data, count);

  return(HPCXX_SUCCESS);
}

int HPCxx_BufferImpl::pack(const u_short* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (u_short*, int)", TAU_HPCXX);
  int err = globus_nexus_check_buffer_size(buffer_,
                                    globus_nexus_sizeof_u_short(count),
                                    globus_nexus_sizeof_u_short(count),
                                    0,
                                    0);
  if (err < 0) return(HPCXX_NO_BUFFER_SPACE);

  globus_nexus_put_u_short(buffer_, (u_short*) data, count);
  
  return(HPCXX_SUCCESS);
}

int HPCxx_BufferImpl::pack(const int* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (int *, int)", TAU_HPCXX);
  int err = globus_nexus_check_buffer_size(buffer_,
                                    globus_nexus_sizeof_int(count),
                                    globus_nexus_sizeof_int(count),
                                    0,
                                    0);
  if (err < 0) return(HPCXX_NO_BUFFER_SPACE);

  globus_nexus_put_int(buffer_, (int*) data, count);

  return(HPCXX_SUCCESS);
}

int HPCxx_BufferImpl::pack(const u_int* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (u_int*, int)", TAU_HPCXX);
  int err = globus_nexus_check_buffer_size(buffer_,
                                    globus_nexus_sizeof_u_int(count),
                                    globus_nexus_sizeof_u_int(count),
                                    0,
                                    0);
  if (err < 0) return(HPCXX_NO_BUFFER_SPACE);

  globus_nexus_put_u_int(buffer_, (u_int*) data, count);

  return(HPCXX_SUCCESS);
}

int HPCxx_BufferImpl::pack(const long* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (long*, int)", TAU_HPCXX);

  int err = globus_nexus_check_buffer_size(buffer_,
                                    globus_nexus_sizeof_long(count),
                                    globus_nexus_sizeof_long(count),
                                    0,
                                    0);
  if (err < 0) return(HPCXX_NO_BUFFER_SPACE);

  globus_nexus_put_long(buffer_, (long*) data, count);

  return(HPCXX_SUCCESS);
}

int HPCxx_BufferImpl::pack(const u_long* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (u_long*, int)", TAU_HPCXX);
  int err = globus_nexus_check_buffer_size(buffer_,
                                    globus_nexus_sizeof_u_long(count),
                                    globus_nexus_sizeof_u_long(count),
                                    0,
                                    0);
  if (err < 0) return(HPCXX_NO_BUFFER_SPACE);

  globus_nexus_put_u_long(buffer_, (u_long*) data, count);

  return(HPCXX_SUCCESS);
}

int HPCxx_BufferImpl::pack(const char* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (char *, int)", TAU_HPCXX);
  int err = globus_nexus_check_buffer_size(buffer_,
                                    globus_nexus_sizeof_char(count),
                                    globus_nexus_sizeof_char(count),
                                    0,
                                    0);
  if (err < 0) return(HPCXX_NO_BUFFER_SPACE);

  globus_nexus_put_char(buffer_, (char*) data, count);

  return(HPCXX_SUCCESS);
}

int HPCxx_BufferImpl::pack(const u_char* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (u_char*, int)", TAU_HPCXX);
  int err = globus_nexus_check_buffer_size(buffer_,
                                    globus_nexus_sizeof_u_char(count),
                                    globus_nexus_sizeof_u_char(count),
                                    0,
                                    0);
  if (err < 0) return(HPCXX_NO_BUFFER_SPACE);

  globus_nexus_put_u_char(buffer_, (u_char*) data, count);

  return(HPCXX_SUCCESS);
}

/*
int HPCxx_BufferImpl::pack(const hpcxx_byte* data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::pack()", "int (hpcxx_byte*, int)", TAU_HPCXX);
  int err = globus_nexus_check_buffer_size(buffer_,
                                    globus_nexus_sizeof_u_char(count),
                                    globus_nexus_sizeof_u_char(count),
                                    0,
                                    0);
  if (err < 0) return(HPCXX_NO_BUFFER_SPACE);

  globus_nexus_put_u_char(buffer_, (u_char*) data->byte, count);

  return(HPCXX_SUCCESS);
}
*/



void HPCxx_BufferImpl::unpack(HPCxx_ContextID *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (HPCxx_ContextID *, int)", 
    TAU_HPCXX);
  globus_nexus_get_startpoint(buffer_, data, count);
}

void HPCxx_BufferImpl::unpack(float *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (float *, int)", TAU_HPCXX);
  globus_nexus_get_float(buffer_, data, count);
}

void HPCxx_BufferImpl::unpack(double *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (double *, int)", TAU_HPCXX);
  globus_nexus_get_double(buffer_, data, count);
}

void HPCxx_BufferImpl::unpack(short *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (short *, int)", TAU_HPCXX);
  globus_nexus_get_short(buffer_, data, count);
}

void HPCxx_BufferImpl::unpack(u_short *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (u_short *, int)", TAU_HPCXX);
  globus_nexus_get_u_short(buffer_, data, count);
}

void HPCxx_BufferImpl::unpack(int *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (int *, int)", TAU_HPCXX);
  globus_nexus_get_int(buffer_, data, count);
}

void HPCxx_BufferImpl::unpack(u_int *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (u_int *, int)", TAU_HPCXX);
  globus_nexus_get_u_int(buffer_, data, count);
}

void HPCxx_BufferImpl::unpack(long *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (long *, int)", TAU_HPCXX);
  globus_nexus_get_long(buffer_, data, count);
}

void HPCxx_BufferImpl::unpack(u_long *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (u_long *, int)", TAU_HPCXX);
  globus_nexus_get_u_long(buffer_, data, count);
}

void HPCxx_BufferImpl::unpack(char *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (char *, int)", TAU_HPCXX);
  globus_nexus_get_char(buffer_, data, count);
}

void HPCxx_BufferImpl::unpack(u_char *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (u_char *, int)", TAU_HPCXX);
  globus_nexus_get_u_char(buffer_, data, count);
}

/*
void HPCxx_BufferImpl::unpack(hpcxx_byte *data, int count)
{
  TAU_PROFILE("HPCxx_BufferImpl::unpack()", "void (hpcxx_byte *, int)", TAU_HPCXX);
  globus_nexus_get_u_char(buffer_, &data->byte, count);
}
*/
