#include <globus_ldap_config.h>
#include <globus_toolkit.h>

#include <stdio.h>
#include <string.h>
#include <ctype.h>
#include <lber.h>
#include <ldap.h>
#include <ldif.h>

#include "ldapconfig.h"

#define DEFSEP		"="

#ifdef LDAP_DEBUG
extern int ldap_debug, lber_debug;
#endif /* LDAP_DEBUG */

static char	*binddn = LDAPSEARCH_BINDDN;
static char	*base = "o=Globus, c=US";
static char	*ldaphost = "mds11.globus.org";
static int	ldapport = 391;
static char	*sep = DEFSEP;
static char *filtpattern="(&(hn=%s)(objectclass=globusservicejobmanager)(contact=*))";
static char *attr="contact";

int getContactString( char *hostname, char *contactString)
{
    char		**attrs;
    int			rc, i, first, scope, deref;
    int			ldap_options, timelimit, sizelimit, authmethod;
    LDAP		*ld;
    extern char		*optarg;
    extern int		optind;

    scope = LDAP_SCOPE_SUBTREE;

	attrs=(char **)malloc(2*sizeof(char *));
	attrs[0]=attr;
	attrs[1]=NULL;


    if (( ld = ldap_open( ldaphost, ldapport )) == NULL ) {
		if (( ld = ldap_open( ldaphost, 390 )) == NULL ) {
			perror( ldaphost );
			exit( 1 );
    	}
	}

    ld->ld_deref = 0;
    ld->ld_timelimit = 0;
    ld->ld_sizelimit = 0;
    ld->ld_options = 0;

	authmethod = LDAP_AUTH_SIMPLE;
    if ( ldap_bind_s( ld, binddn, NULL, authmethod ) != LDAP_SUCCESS ) {
	ldap_perror( ld, "ldap_bind" );
	exit( 1 );
    }

	rc = dosearch( ld, base, scope, attrs, hostname, contactString );

    ldap_unbind( ld );

	return(rc);

}

dosearch( LDAP *ld, char *base, int scope, char **attrs, char *value, char *contactString )
{
    char		filter[ BUFSIZ ], **val;
    int			rc, first;
    LDAPMessage		*res=(LDAPMessage *)NULL, *e;
    char		*a;
    BerElement		*ber;
    struct berval	**bvals=(struct berval **)NULL;

    sprintf( filter, filtpattern, value );

    if ( ldap_search( ld, base, scope, filter, attrs, 0 ) == -1 ) {
		ldap_perror( ld, "ldap_search" );
		return( ld->ld_errno );
    }

    while ( (rc = ldap_result( ld, LDAP_RES_ANY, 0, NULL, &res ))
	    == LDAP_RES_SEARCH_ENTRY ) {
	e = ldap_first_entry( ld, res );
    a = ldap_first_attribute( ld, e, &ber ); 
	if(( bvals = ldap_get_values_len( ld, e, a )) != NULL ) {
		sprintf(contactString, "%s", bvals[0]->bv_val);
		}
	if(bvals!=(struct berval **)NULL) ber_bvecfree( bvals );
	if(res!=(LDAPMessage *)NULL) ldap_msgfree( res );
    }
    if ( rc == -1 ) {
	ldap_perror( ld, "ldap_result" );
	return( rc );
    }
    if (( rc = ldap_result2error( ld, res, 0 )) != LDAP_SUCCESS ) {
        ldap_perror( ld, "ldap_search" );
    }

    if(res!=(LDAPMessage *)NULL) ldap_msgfree( res );
    return( rc );
}
