//-----------------------------------------------------------------------------
// HPCxx_Comm.C -- Category of objects used for communication.  For example,
// Remote Method Invocation (RMI) and global pointer get/put.
//-----------------------------------------------------------------------------

#include "HPCxx_Comm.h"
#include "hpcxx_rtsImpl.h"
#include <iostream.h>


static const int DEFAULT_BUF_SIZE = 128;


HPCxx_Comm::HPCxx_Comm(HPCxx_Mutex* mutex)
{
  waiting_ = false;
  mutex_   = mutex;
  buffer_  = NULL;  
}


//---- wait -------------------------------------------------------------------
// Waits until the communication operation has completed (or at least
// until buffers can be reused).  The function blocks until the signal
// method on this object is called.
//-----------------------------------------------------------------------------
void HPCxx_Comm::wait(void)
{
  condMutex_.lock();
  waiting_ = true;
  while (waiting_) cond_.wait(condMutex_);
  condMutex_.unlock();
}


//---- signal -----------------------------------------------------------------
// Signals that the communication operation has completed and allows threads
// waiting on completion of the communication to continue execution.
//-----------------------------------------------------------------------------
void HPCxx_Comm::signal(void)
{
  condMutex_.lock();
  if (waiting_) cond_.broadcast();
  waiting_ = false;
  condMutex_.unlock();
}


ostream& operator<<(ostream& s, const HPCxx_Comm& c)
{
  return s << "tag = " << (void*) c.messageTag() << ", sender = " << c.sender_
           << ", dest = " << c.destination_
           << ", size = " << c.messageBuffer().impl_.size()
           << ", type = " << c.messageType() << ", id = " << c.messageID();
//           << ", thread = " << c.messageThread();
}

